// Nancy Montanez
// Connection file (with accept, listen, and other helpers.)
// Created: Sometime in October
// Modified: Nov. 26, 2000
// Modified again: Dec 2, 2000
// Modified again++: Jan 30, 2001 (Let this be the last time!)
// Modified again+2: Feb 2, 2001 (PLease!!!)


#define _WIN32_WINNT 0x0400
#define WINVER 0x0400

#include <windows.h>
#include <ws2tcpip.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
//#include "dmzcore.h"
#include "dmz_net.h"
#include "dmz_net_internal.h"

//------------------------------
int dmz_avail_socket(void)
{
	
	int i;

	// Enter the critical section
	EnterCriticalSection(&cs);

	for(i = 1; i < MAX_CONNECT; i++)
	{
		if(sockets[i].s == INVALID_SOCKET)
				break;
	}
	
	//Leave the critical section
	LeaveCriticalSection(&cs);

	if (i == MAX_CONNECT)
		return DMZ_NET_TOO_MANY_SOCS;
	else 
		return i;
}

//------------------------------------
int dmz_connect(char *peer, int port)
{
	//struct sockaddr_in sa;
	int nRet;
	int sock_index;

	if (!init_done)
		return DMZ_NET_NOT_INIT;

	

	// check for available sockets in the array
	
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;
	
	//Check for invalid peers sent in by the function call
	if ( (peer == NULL) || (port <= 0) )
		return DMZ_NET_INVALID_PARAM;
	
	
	sockets[sock_index].sa.sin_family = PF_INET;
	sockets[sock_index].sa.sin_addr.s_addr = inet_addr(peer);
	sockets[sock_index].sa.sin_port = htons(port);

	sockets[sock_index].s = socket(PF_INET, SOCK_STREAM, 0);

	if( sockets[sock_index].s == INVALID_SOCKET) 
		return DMZ_NET_SOCKET_ERROR;

	// now, call WinSock's connect to initiate the connection 
	nRet = connect(sockets[sock_index].s, (const struct sockaddr *) &sockets[sock_index].sa, sizeof(struct sockaddr_in));
	
	if( nRet == SOCKET_ERROR)
		return DMZ_NET_NO_CONNECT;

	return sock_index;
}

//-------------------------------------------
//Function: dmz_listen
//
int dmz_listen(int port, int max)
{
	int nRet;
	int sock_index;
	int list_ret;

	if (!init_done)
		return DMZ_NET_NOT_INIT;


	// check for available sockets in the array
	
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;

	//Check the parameters that are passed in from the function
	
	if( (port <= 0) || (max <= 0) || (max > MAX_CONNECT) )
		return DMZ_NET_INVALID_PARAM;
	
	sockets[sock_index].s = socket(PF_INET, SOCK_STREAM, 0);

	if( sockets[sock_index].s == INVALID_SOCKET) 
		return DMZ_NET_SOCKET_ERROR;
	

	sockets[sock_index].sa.sin_family = AF_INET;
	sockets[sock_index].sa.sin_addr.s_addr = htonl(INADDR_ANY);
	sockets[sock_index].sa.sin_port = htons(port);

	// now, name the socket using bind 
	nRet = bind(sockets[sock_index].s, (const struct sockaddr *) &sockets[sock_index].sa, sizeof(struct sockaddr_in));
	if( nRet == SOCKET_ERROR)
		return DMZ_NET_BINDING_ERR;


	list_ret = listen(sockets[sock_index].s, max);
	
	if (list_ret == SOCKET_ERROR)
		return DMZ_NET_LISTEN_ERR;


	return sock_index;
}
//----------------------------------------
//Function: dmz_accept
//

int dmz_accept(int listen_index)
{
	SOCKET accepted;
	int z = sizeof(struct sockaddr_in);

	int sock_index;

	if (!init_done)
		return DMZ_NET_NOT_INIT;

	
	if ((listen_index < 0) || (listen_index > MAX_CONNECT))
		return DMZ_NET_INVALID_PARAM;

	// check for available sockets in the array
	
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;
	
	
	// once we get some non-zero number, we can accept. 
	accepted = accept(sockets[listen_index].s, (struct sockaddr *) &sockets[sock_index].sa, &z);

	if (accepted == INVALID_SOCKET)
		return DMZ_NET_ACCEPT_ERR;

	sockets[sock_index].s = accepted;

	return sock_index;
}

//---------------------------------------------------
//Function: dmz_disconnect
//
void dmz_disconnect(int id)
{
	
	closesocket(sockets[id].s);

	sockets[id].s = INVALID_SOCKET;

}
//----------------------------------------------------
//Function: dmz_multiplex
int dmz_net_multiplex(int *read_array, int *write_array, int *accept_array,
							   int *read_ids, int *write_ids, int *accept_ids,
							   struct timeval *timeout)
{
	//variables to be used for the select function call
	// *_return contains a copy of the arrays
	fd_set read_set;
	fd_set write_set;
	fd_set error_set;
	
	int *read_arraycp;
	int *write_arraycp; 
	int	*accept_arraycp;

	//variable for fun
	int x,i,j,z,y,WSAErr;
	int rv;				// return value from the select() call
	int error = 0;
	int error_temp = 0;
//	char buf[256];

	// check to see if the params that are sent in are correct
	//error =  (( (int) read_array ^ (int) read_ids) || ( (int) accept_array ^ (int) accept_ids) ||
	//	((int) write_array ^ (int) write_ids));
	
	if ( ((read_array == NULL) && (read_ids != NULL))  || ((read_array != NULL) && (read_ids == NULL)))
		error = 1;

	else if ( ((write_array == NULL) && (write_ids != NULL))  || ((write_array != NULL) && (write_ids == NULL)))
		error = 1;

	else if ( ((accept_array == NULL) && (accept_ids != NULL))  || ((accept_array != NULL) && (accept_ids == NULL)))
		error = 1;

	// parameters are not correct, return back to calling function

	if (error) return DMZ_NET_INVALID_PARAM;

	// copy the arrays into new arrays to preserve the original contents of the errors
	// first, set space for the new arrays

	if (read_array != NULL)
		read_arraycp = (int *) malloc (sizeof(int) * (*read_ids));

	if (accept_array != NULL)
		accept_arraycp = (int *) malloc (sizeof(int) * (*accept_ids));

	if (write_array != NULL)
		write_arraycp = (int *) malloc (sizeof(int) * (*write_ids));
	
	FD_ZERO(&read_set);
	FD_ZERO(&write_set);
	FD_ZERO(&error_set);

	// set the read array
	if(read_array != NULL)
	{
		for(i = 0; i < *read_ids; i++)
			FD_SET (sockets[read_array[i]].s, &read_set);
	}

	// set the accept array
	if (accept_array != NULL)
	{
		for(i = 0; i < *accept_ids; i++)
			FD_SET (sockets[accept_array[i]].s, &read_set);

	}
	// set the accept array
	if (write_array != NULL)
	{
		for(i = 0; i < *write_ids; i++)
			FD_SET (sockets[write_array[i]].s, &write_set);
	}

	//before calling select, copy the arrays into the temp arrays

	if (read_ids)
		memcpy( (void *) read_arraycp, (void *) read_array, (sizeof(int) * (*read_ids)) );
	if (accept_ids)
		memcpy( (void *) accept_arraycp, (void *) accept_array, (sizeof(int) * (*accept_ids)) );
	if (write_ids)
		memcpy( (void *) write_arraycp, (void *) write_array, (sizeof(int) * (*read_ids)) );
	
	// Now that the arrays are set, call the function.
	rv = select(0, &read_set, &write_set, &error_set, NULL);
	
//	sprintf(buf, "return value of select: %d", rv);
//	dmz_logstd(buf);

	//Check the return val from select()
	if (rv == 0)
		return DMZ_NET_TRY_AGAIN;

	if (rv == SOCKET_ERROR)
	{
		WSAErr = WSAGetLastError();
		
//		sprintf(buf,"WSAErr = %d", WSAErr);
//		dmz_logstd(buf);
		return DMZ_NET_SOCKET_ERROR;
	}
	
	z = 0;

	if (read_ids)
	{
		for (j = 0; j < *read_ids; j++)
		{
			if(FD_ISSET(sockets[read_arraycp[j]].s, & read_set))
				read_array[z++] = read_arraycp[j];
		}
		*read_ids = z;
	}


	y = 0;
	if (accept_ids)
	{
		
		for (j = 0; j < *accept_ids; j++)
		{
			if(FD_ISSET(sockets[accept_arraycp[j]].s, & read_set))
				accept_array[y++] = accept_arraycp[j];
		}	
		*accept_ids = y;
	}
	x = 0;

	if(write_ids)
	{

		for (j = 0; j < *write_ids; j++)
		{
			if(FD_ISSET(sockets[write_arraycp[j]].s, & write_set))
				write_array[x++] = write_arraycp[j];
		}
		*write_ids = x;
	}
	
	
	// return the total number of sockets that are ready
	return x+y+z;


}
//-------------------------------------------------------------------
// Function dmz_net_chmode
// return value is 0 for blocking, 1 for nonblocking
int dmz_net_chmode(int connid, DMZ_SOCKET_MODE mode)
{
	u_long argp = 1L;			/* nonzero enables nonblocking mode */
	int nRet;

	if ( (connid < 0) || (connid > MAX_CONNECT))
		return DMZ_NET_INVALID_PARAM;

	if(mode == NONBLOCKING)
		argp = 1L;
	
	else if (mode == BLOCKING)
		argp = 0L;

	else 
		return DMZ_NET_INVALID_PARAM;

	nRet = ioctlsocket(sockets[connid].s, FIONBIO, (u_long FAR *) &argp);
	
	if (nRet == SOCKET_ERROR)
		return DMZ_NET_NONBLOCKING_ERR;

	return (sockets[connid].m = mode);

	
}

//-------------------------------------------------------------------
// Function dmz_net_getpeer
char *dmz_net_getpeer(int connid)
{
	int z = sizeof(struct sockaddr_in);
	int gpn_ret = 0;
	//struct sockaddr_in joe;
	char *tempipaddr;
	
	if ((connid < 0) || (connid > MAX_CONNECT) || (sockets[connid].s == INVALID_SOCKET))
		return NULL;

	// retrieve the current name of the peer connected to a socket
	//gpn_ret = getpeername(sockets[connid].s, (struct sockaddr*) &joe, &z);

	//if (gpn_ret != 0)
	//	return NULL;

	//tempipaddr = inet_ntoa((struct in_addr) sockets[connid].sa.sin_addr);
	
	tempipaddr = inet_ntoa(sockets[connid].sa.sin_addr);
	//if(tempipaddr == NULL)
		//return DMZ_NET_SOCKET_ERROR;

	//strcpy(ip_addr, tempipaddr);

	return tempipaddr;


} 

int dmz_net_get_local_IP(char *buffer)
{
	int i, numLocalAddr, wsError;
	char *pAddrString;
	DWORD bytesReturned;

	SOCKET s;
	SOCKADDR_IN *pAddrInet;
	INTERFACE_INFO localAddr[10];

	s = WSASocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, 0, 0);
	if (s == INVALID_SOCKET)
	{
		WSACleanup();
		return DMZ_NET_SOCKET_ERROR;
   	}

	wsError = WSAIoctl(s, SIO_GET_INTERFACE_LIST, NULL, 0, &localAddr,
                      sizeof(localAddr), &bytesReturned, NULL, NULL);

	if (wsError == SOCKET_ERROR)
	{
//		last_error = GetLastError();
		return DMZ_NET_SOCKET_ERROR;
	}

	closesocket(s);

	// Display interface information
	numLocalAddr = (bytesReturned / sizeof(INTERFACE_INFO));
	for (i = 0; i < numLocalAddr; i++) 
	{
		pAddrInet = (SOCKADDR_IN *) &localAddr[i].iiAddress;
		pAddrString = inet_ntoa(pAddrInet->sin_addr);
		if (pAddrString)
		{
			strcpy(buffer, pAddrString);
			break;
		}
	}

	return DMZ_NET_SUCCESS;
}

//-------------------END OF conn.cpp----------------------------------------------------
