//-----------------------------------------------------------------------------
// File: dmz_graphics_test.CPP
// Date: 10/18/2000 2:21 am
// Written by: Emory Al-Imam
//
// Desc: This is a test program which displays a bitmap using the DMZ graphics
//		 library.  The test program runs at 800x600 in 32-bit color and uses
//		 page-flipping to display the bitmap using one off-screen back buffer.
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>
#include "resource.h"

#include "dmzgraph.h"

//-----------------------------------------------------------------------------
// Local definitions
//-----------------------------------------------------------------------------

// Misc. info about application
#define NAME                "dmz_graphics_lib_test"
#define TITLE               "DMZ Graphics Library Test"

BOOL                        g_bActive = FALSE;   // Is application active?

int mybitmap, mybitmap2;					// dmz bitmap ID
int width, height;

//-----------------------------------------------------------------------------
// Name: UpdateFrame()
// Desc: Decide what needs to be blitted next, wait for blit to complete,
//       then flip the buffers.
//-----------------------------------------------------------------------------
void UpdateFrame(void)
{
	int dx = 800 / 2 - width / 2;
	int dy = 600 / 2 - height / 2;

	int sx = 0;
	int sy = 0 + height / 2;

	int w = width / 2;
	int h = height / 2;

	dmz_graph_blit(mybitmap, sx, sy, w, h, dx, dy, 0x80);
/*	rv = dmz_graph_setpixel(mybitmap, 10, 10, 0xFF0000);
	rv = dmz_graph_setpixel(mybitmap, 11, 10, 0xFF0000);
	rv = dmz_graph_setpixel(mybitmap, 12, 10, 0xFF0000);

	sprintf(buf, "set pixel to image returned %d", rv);
	dmz_logstd(buf);
*/
//	dmz_graph_blit(mybitmap, 0, 0, width, height, dx, dy, -1);
/*
	rv = dmz_graph_setpixel(0, 100, 100, 0xFF00);
	rv = dmz_graph_setpixel(0, 101, 100, 0xFF00);
	rv = dmz_graph_setpixel(0, 102, 100, 0xFF00);

	sprintf(buf, "set pixel to screen returned %d", rv);
	dmz_logstd(buf);
*/
	dmz_graph_flip();
}

//-----------------------------------------------------------------------------
// Name: WindowProc()
// Desc: The Main Window Procedure
//-----------------------------------------------------------------------------
long FAR PASCAL
WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_ACTIVATE:
            // Pause if minimized
            g_bActive = !((BOOL)HIWORD(wParam));
            return 0L;

        case WM_DESTROY:
            // Clean up and close the app
			dmz_graph_kill();
            PostQuitMessage(0);
            return 0L;

        case WM_KEYDOWN:
            // Handle any non-accelerated key commands
            switch (wParam)
            {
                case VK_ESCAPE:
                case VK_F12:
                    PostMessage(hWnd, WM_CLOSE, 0, 0);
                    return 0L;
            }
            break;

        case WM_SETCURSOR:
            // Turn off the cursor since this is a full-screen app
            SetCursor(NULL);
            return TRUE;
    }
    return DefWindowProc(hWnd, message, wParam, lParam);
}

//-----------------------------------------------------------------------------
// Name: InitApp()
// Desc: Do work required for every instance of the application:
//          Create the window, initialize data
//-----------------------------------------------------------------------------
static HRESULT InitApp(HINSTANCE hInstance, int nCmdShow)
{
    HWND                        hWnd;
    WNDCLASS                    wc;

    // Set up and register window class
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAIN_ICON));
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH )GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass(&wc);

    // Create a window
    hWnd = CreateWindowEx(WS_EX_TOPMOST,
                          NAME,
                          TITLE,
                          WS_POPUP,
                          0,
                          0,
                          GetSystemMetrics(SM_CXSCREEN),
                          GetSystemMetrics(SM_CYSCREEN),
                          NULL,
                          NULL,
                          hInstance,
                          NULL);
    if (!hWnd)
        return FALSE;
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);
    SetFocus(hWnd);

	int z;

	// Initialize dmz graphics library
	z = dmz_graph_init(hWnd);

	// sets display and also creates primary and back surfaces which are size of display specified
	z = dmz_graph_set_mode(800, 600, 16);

	// load a bitmap file to an offscreen surface and return a handle it
	mybitmap = dmz_graph_load_image("dmz_map.bmp", &width, &height);

    return 0;
}

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Initialization, message loop
//-----------------------------------------------------------------------------
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;

    // function that initializes all DirectX components/objects
	if (InitApp(hInstance, nCmdShow) != 0)
        return FALSE;

    while (TRUE)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                return msg.wParam;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else if (g_bActive)
			UpdateFrame();
        
        else
            WaitMessage();  // Make sure we go to sleep if we have nothing else to do
    }
}