/*	File:	dmz_snd.h 
	Author:	Alex Fedosov
	Date:	October 9, 2000
	Update:	October 16, 2000
	Description:	DMZ sound API
*/

#ifndef __DMZ_SND_H__
#define __DMZ_SND_H__

#ifdef __cplusplus
	#ifdef DMZ_SND_EXPORTS
		#define DMZ_SOUND_API extern "C" __declspec(dllexport)
	#else
		#define DMZ_SOUND_API extern "C" __declspec(dllimport)
	#endif
#else
	#ifdef DMZ_SND_EXPORTS
		#define DMZ_SOUND_API __declspec(dllexport)
	#else
		#define DMZ_SOUND_API __declspec(dllimport)
	#endif
#endif

/* constant values */

#define DMZ_SND_LOOP 1


/* function prototypes */

DMZ_SOUND_API int dmz_snd_init(void* hwnd);
DMZ_SOUND_API int dmz_snd_load(char* file);
DMZ_SOUND_API int dmz_snd_play(int sid, int flags);
DMZ_SOUND_API int dmz_snd_stop(int sid);
DMZ_SOUND_API int dmz_snd_unload(int sid);
DMZ_SOUND_API int dmz_snd_get_volume(int sid, int *volume);
DMZ_SOUND_API int dmz_snd_set_volume(int sid, int volume);
DMZ_SOUND_API int dmz_snd_set_pan(int sid, int pan);
DMZ_SOUND_API int dmz_snd_get_pan(int sid, int *pan);
DMZ_SOUND_API void dmz_snd_kill(void);


/* error codes */
typedef enum {
	DMZ_SND_SUCCESS		=  0,		// operation completed successfully
	DMZ_SND_INVALID_SID	= -1,		// supplied sound id is out of range
	DMZ_SND_UNUSED_SID	= -2,		// no sound loaded under this sound id
	DMZ_SND_TOO_MANY	= -3,		// too many sounds loaded
	DMZ_SND_NOT_INIT	= -4,		// library has not yet been initialized
	DMZ_SND_ALREADY_INIT = -5,		// library has already been initialized
	DMZ_SND_BAD_VOLUME	= -6,		// invalid volume supplied
	DMZ_SND_BAD_PAN		= -7,		// invalid pan supplied

	DMZ_SND_DS_ERROR	= -101,		// DirectSound error (Windows only)

	DMZ_SND_OPEN_ERROR	= -201,		// error opening file
	DMZ_SND_READ_ERROR	= -202,		// error reading file
	DMZ_SND_BAD_WAVE	= -203,		// not a valid WAV file
	DMZ_SND_NO_MEM		= -204		// error allocating memory
} DMZ_SND_RV;

#endif
