/* test of the dmz sound library */

#include <windows.h>
#include <commctrl.h>
#include <cstring>
#include <cstdio>
#include <cctype>

#include "resource.h"
#include "menu.h"
#include "dmz_snd.h"

LRESULT CALLBACK WindowFunc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DialogFunc(HWND hdwnd, UINT message, WPARAM wparam, LPARAM lparam);

char szWinName[] = "dmzsndtest";
HINSTANCE hInst;

int WINAPI WinMain(HINSTANCE hThisInst, HINSTANCE hPrevInst,
				   LPSTR lpszArgs, int nWinMode)
{
	HWND hwnd;
	MSG msg;
	WNDCLASSEX wcl;
	INITCOMMONCONTROLSEX cc;
	
	/* define a window class */
	wcl.cbSize = sizeof(WNDCLASSEX);

	wcl.hInstance = hThisInst;
	wcl.lpszClassName = szWinName;
	wcl.lpfnWndProc = WindowFunc;
	wcl.style = 0;

	wcl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wcl.hIconSm = LoadIcon(NULL, IDI_WINLOGO);
	wcl.hCursor = LoadCursor(NULL, IDC_ARROW);

	wcl.lpszMenuName = "MENU_SNDTEST";
	wcl.cbClsExtra = 0;
	wcl.cbWndExtra = 0;

	wcl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);

	/* register the window class */
	if (!RegisterClassEx(&wcl)) return 0;

	hInst = hThisInst;

	/* window class registered, now create it */
	hwnd = CreateWindow(
		szWinName,
		"dmz sound test",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		HWND_DESKTOP,
		NULL,
		hThisInst,
		NULL
	);

	// initialize the common controls
	cc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	cc.dwICC = ICC_BAR_CLASSES;
	InitCommonControlsEx(&cc);

	/* display the window */
	ShowWindow(hwnd, nWinMode);

	UpdateWindow(hwnd);

	/* create the message loop */
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

LRESULT CALLBACK WindowFunc(HWND hwnd, UINT message,
							WPARAM wparam, LPARAM lparam)
{
	static int wavid;
	OPENFILENAME ofn;

	static char fn[256];
	char filefilter[] = "All Files\0*.*\0\0\0";
	
	switch (message)
	{
	case WM_CREATE:
		dmz_snd_init(hwnd);
		DialogBox(hInst, "SNDTEST", hwnd, (DLGPROC) DialogFunc);
		DestroyWindow(hwnd);
		break;

	case WM_COMMAND:
		switch (LOWORD(wparam))
		{
		case ID_FILE_LOAD:
			memset(&ofn, 0, sizeof(OPENFILENAME));
			ofn.lStructSize = sizeof(OPENFILENAME);
			ofn.hwndOwner = hwnd;
			ofn.lpstrFilter = filefilter;
			ofn.nFilterIndex = 1;
			ofn.lpstrFile = fn;
			ofn.nMaxFile = sizeof(fn);
			ofn.lpstrFileTitle = NULL;
			ofn.lpstrTitle = "Load a WAV file...";
			ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

			if (!GetOpenFileName(&ofn))
				break;

			if ((wavid = dmz_snd_load(fn)) < 0)
				MessageBox(hwnd, "Error loading file!", "Error", MB_OK);
			break;
		
		case ID_FILE_PLAY:
			if (dmz_snd_play(wavid, 0) < 0)
				MessageBox(hwnd, "Error playing sound!", "Error", MB_OK);
			break;

		case ID_FILE_STOP:
			dmz_snd_stop(wavid);
			break;

		case ID_HELP_ABOUT:
			MessageBox(hwnd, "dmz_sound test\nTeam Overclocked 2000", "About", MB_OK);
			break;
		
		case ID_FILE_EXIT:
			DestroyWindow(hwnd);
			break;

		default:
			break;
		}
		break;
	
	case WM_DESTROY:
		dmz_snd_unload(wavid);
		dmz_snd_kill();
		PostQuitMessage(0);
		break;
		
	default:
		return DefWindowProc(hwnd, message, wparam, lparam);
	}

	return 0;
}

BOOL CALLBACK DialogFunc(HWND hdwnd, UINT message, WPARAM wparam, LPARAM lparam)
{
	static int wavid;
	OPENFILENAME ofn;
	int volume, pan, rv;

	static char fn[256];
	char ft[64];
	char filefilter[] = "All Files\0*.*\0\0\0";
	char buf[10];

	switch (message)
	{
	case WM_COMMAND:
		switch (LOWORD(wparam))
		{
		case ID_BTN_LOAD:
			dmz_snd_init(hdwnd);
			memset(&ofn, 0, sizeof(OPENFILENAME));
			ofn.lStructSize = sizeof(OPENFILENAME);
			ofn.hwndOwner = hdwnd;
			ofn.lpstrFilter = filefilter;
			ofn.nFilterIndex = 1;
			ofn.lpstrFile = fn;
			ofn.nMaxFile = sizeof(fn);
			ofn.lpstrFileTitle = ft;
			ofn.nMaxFileTitle = sizeof(ft);
			ofn.lpstrTitle = "Load a WAV file...";
			ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

			if (!GetOpenFileName(&ofn))
				break;

			if ((wavid = dmz_snd_load(fn)) < 0)
				MessageBox(hdwnd, "Error loading file!", "Error", MB_OK);
			else
				SetDlgItemText(hdwnd, IDC_FILENAME, ft);
			
			break;

		case ID_BTN_UNLOAD:
			if (wavid >= 0) dmz_snd_unload(wavid);
			SetDlgItemText(hdwnd, IDC_FILENAME, "");
			break;

		case ID_BTN_PLAY:
			if (dmz_snd_play(wavid, 0) < 0)
				MessageBox(hdwnd, "Error playing sound!", "Error", MB_OK);
			break;

		case ID_BTN_STOP:
			if (wavid >= 0) dmz_snd_stop(wavid);
			break;

		case ID_BTN_SETVOL:
			GetDlgItemText(hdwnd, IDC_VOLUME, buf, 8);
			volume = atoi(buf);
			rv = dmz_snd_set_volume(wavid, volume);

			if (rv < 0)
			{
				char error[100];
				sprintf(error, "Error setting volume to %d, returned %d", volume, rv);
				MessageBox(hdwnd, error, "Error", MB_OK);
			}
			break;

		case ID_BTN_GETVOL:
			if (dmz_snd_get_volume(wavid, &volume) < 0)
				MessageBox(hdwnd, "Error getting volume!", "Error", MB_OK);
			else
			{
				sprintf(buf, "%d", volume);
				SetDlgItemText(hdwnd, IDC_VOLUME, buf);
			}
			break;

		case ID_BTN_SETPAN:
			GetDlgItemText(hdwnd, IDC_PAN, buf, 8);
			pan = atoi(buf);
			rv = dmz_snd_set_pan(wavid, pan);

			if (rv < 0)
			{
				char error[100];
				sprintf(error, "Error setting pan to %d, returned %d", pan, rv);
				MessageBox(hdwnd, error, "Error", MB_OK);
			}
			break;

		case ID_BTN_GETPAN:
			if (dmz_snd_get_pan(wavid, &pan) < 0)
				MessageBox(hdwnd, "Error getting pan!", "Error", MB_OK);
			else
			{
				sprintf(buf, "%d", pan);
				SetDlgItemText(hdwnd, IDC_PAN, buf);
			}
			break;

		case ID_BTN_EXIT:
			dmz_snd_kill();
			EndDialog(hdwnd, 0);
			return 1;
		}
	}

	return 0;
}
